@@ -157,11 +157,13 @@ bigquery_dialect.add(
             Ref("BaseExpressionElementGrammar"),
         ),
     ),
-    ExtractFunctionName=StringParser(
-        "EXTRACT",
-        SymbolSegment,
-        name="function_name_identifier",
-        type="function_name_identifier",
+    ExtendedDatetimeUnitSegment=SegmentGenerator(
+        lambda dialect: RegexParser(
+            r"^(" + r"|".join(dialect.sets("extended_datetime_units")) + r")$",
+            CodeSegment,
+            name="date_part",
+            type="date_part",
+        )
     ),
 )
 
@@ -197,6 +199,30 @@ bigquery_dialect.replace(
             bracket_pairs_set="angle_bracket_pairs",
         ),
     ),
+    StructTypeGrammar=Sequence(
+        "STRUCT",
+        Bracketed(
+            Delimited(  # Comma-separated list of field names/types
+                Sequence(
+                    OneOf(
+                        # ParameterNames can look like Datatypes so can't use
+                        # Optional=True here and instead do a OneOf in order
+                        # with DataType only first, followed by both.
+                        Ref("DatatypeSegment"),
+                        Sequence(
+                            Ref("ParameterNameSegment"),
+                            Ref("DatatypeSegment"),
+                        ),
+                    ),
+                    Ref("OptionsSegment", optional=True),
+                ),
+                delimiter=Ref("CommaSegment"),
+                bracket_pairs_set="angle_bracket_pairs",
+            ),
+            bracket_type="angle",
+            bracket_pairs_set="angle_bracket_pairs",
+        ),
+    ),
     # BigQuery allows underscore in parameter names, and also anything if quoted in
     # backticks
     ParameterNameSegment=OneOf(
@@ -205,7 +231,12 @@ bigquery_dialect.replace(
         ),
         RegexParser(r"`[^`]*`", CodeSegment, name="parameter", type="parameter"),
     ),
-    DateTimeLiteralGrammar=Nothing(),
+    DateTimeLiteralGrammar=Sequence(
+        OneOf("DATE", "DATETIME", "TIME", "TIMESTAMP"),
+        NamedParser(
+            "single_quote", CodeSegment, name="date_constructor_literal", type="literal"
+        ),
+    ),
     JoinLikeClauseGrammar=Sequence(
         AnyNumberOf(
             Ref("FromPivotExpressionSegment"),
@@ -263,6 +294,9 @@ bigquery_dialect.sets("datetime_units").update(
     ]
 )
 
+# Add additional datetime units only recognised in some functions (e.g. extract)
+bigquery_dialect.sets("extended_datetime_units").update(["DATE", "DATETIME", "TIME"])
+
 bigquery_dialect.sets("date_part_function_name").clear()
 bigquery_dialect.sets("date_part_function_name").update(
     [
@@ -354,7 +388,12 @@ class StatementSegment(ansi.StatementSegment):
 
     match_grammar = ansi.StatementSegment.match_grammar
     parse_grammar = ansi.StatementSegment.parse_grammar.copy(
-        insert=[Ref("DeclareStatementSegment"), Ref("SetStatementSegment")],
+        insert=[
+            Ref("DeclareStatementSegment"),
+            Ref("SetStatementSegment"),
+            Ref("ExportStatementSegment"),
+            Ref("CreateExternalTableStatementSegment"),
+        ],
     )
 
 
@@ -398,7 +437,6 @@ bigquery_dialect.replace(
     LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
         insert=[
             Ref("DoubleQuotedLiteralSegment"),
-            Ref("LiteralCoercionSegment"),
             Ref("ParameterizedSegment"),
         ]
     ),
@@ -435,12 +473,41 @@ bigquery_dialect.replace(
 class ExtractFunctionNameSegment(BaseSegment):
     """EXTRACT function name segment.
 
-    Need to be able to specify this as type function_name
-    so that linting rules identify it properly
+    Need to be able to specify this as type `function_name_identifier`
+    within a `function_name` so that linting rules identify it properly.
     """
 
     type = "function_name"
-    match_grammar: Matchable = Ref("ExtractFunctionName")
+    match_grammar: Matchable = StringParser(
+        "EXTRACT",
+        CodeSegment,
+        name="function_name_identifier",
+        type="function_name_identifier",
+    )
+
+
+class NormalizeFunctionNameSegment(BaseSegment):
+    """NORMALIZE function name segment.
+
+    Need to be able to specify this as type `function_name_identifier`
+    within a `function_name` so that linting rules identify it properly.
+    """
+
+    type = "function_name"
+    match_grammar: Matchable = OneOf(
+        StringParser(
+            "NORMALIZE",
+            CodeSegment,
+            name="function_name_identifier",
+            type="function_name_identifier",
+        ),
+        StringParser(
+            "NORMALIZE_AND_CASEFOLD",
+            CodeSegment,
+            name="function_name_identifier",
+            type="function_name_identifier",
+        ),
+    )
 
 
 class FunctionSegment(ansi.FunctionSegment):
@@ -452,75 +519,98 @@ class FunctionSegment(ansi.FunctionSegment):
     for our purposes.
     """
 
-    match_grammar = OneOf(
-        Sequence(
-            # BigQuery EXTRACT allows optional TimeZone
-            Ref("ExtractFunctionNameSegment"),
-            Bracketed(
-                Ref("DatetimeUnitSegment"),
-                "FROM",
-                Ref("ExpressionSegment"),
-                Ref("TimeZoneGrammar", optional=True),
+    match_grammar = Sequence(
+        # BigQuery Function names can be prefixed by the keyword SAFE to
+        # return NULL instead of error.
+        # https://cloud.google.com/bigquery/docs/reference/standard-sql/functions-reference#safe_prefix
+        Sequence("SAFE", Ref("DotSegment"), optional=True),
+        OneOf(
+            Sequence(
+                # BigQuery EXTRACT allows optional TimeZone
+                Ref("ExtractFunctionNameSegment"),
+                Bracketed(
+                    OneOf(
+                        Ref("DatetimeUnitSegment"),
+                        Ref("ExtendedDatetimeUnitSegment"),
+                    ),
+                    "FROM",
+                    Ref("ExpressionSegment"),
+                ),
             ),
-        ),
-        Sequence(
-            # Treat functions which take date parts separately
-            # So those functions parse date parts as DatetimeUnitSegment
-            # rather than identifiers.
-            Ref(
-                "DatePartFunctionNameSegment",
-                exclude=Ref("ExtractFunctionNameSegment"),
-            ),
-            Bracketed(
-                Delimited(
-                    Ref("DatetimeUnitSegment"),
-                    Ref(
-                        "FunctionContentsGrammar",
-                        ephemeral_name="FunctionContentsGrammar",
+            Sequence(
+                # BigQuery NORMALIZE allows optional normalization_mode
+                # https://cloud.google.com/bigquery/docs/reference/standard-sql/functions-and-operators#normalize
+                Ref("NormalizeFunctionNameSegment"),
+                Bracketed(
+                    Ref("ExpressionSegment"),
+                    Sequence(
+                        Ref("CommaSegment"),
+                        OneOf("NFC", "NFKC", "NFD", "NFKD"),
+                        optional=True,
                     ),
                 ),
             ),
-        ),
-        Sequence(
             Sequence(
+                # Treat functions which take date parts separately
+                # So those functions parse date parts as DatetimeUnitSegment
+                # rather than identifiers.
                 Ref(
-                    "FunctionNameSegment",
-                    exclude=OneOf(
-                        Ref("DatePartFunctionNameSegment"),
-                        Ref("ValuesClauseSegment"),
-                    ),
+                    "DatePartFunctionNameSegment",
+                    exclude=Ref("ExtractFunctionNameSegment"),
                 ),
                 Bracketed(
-                    Ref(
-                        "FunctionContentsGrammar",
-                        # The brackets might be empty for some functions...
-                        optional=True,
-                        ephemeral_name="FunctionContentsGrammar",
-                    )
-                ),
-            ),
-            # Functions returning ARRAYS in BigQuery can have optional
-            # Array Accessor clauses
-            Ref("ArrayAccessorSegment", optional=True),
-            # Functions returning STRUCTs in BigQuery can have the fields
-            # elements referenced (e.g. ".a"), including wildcards (e.g. ".*")
-            # or multiple nested fields (e.g. ".a.b", or ".a.b.c")
-            Sequence(
-                Ref("DotSegment"),
-                AnyNumberOf(
-                    Sequence(
-                        Ref("ParameterNameSegment"),
-                        Ref("DotSegment"),
+                    Delimited(
+                        Ref("DatetimeUnitSegment"),
+                        Ref(
+                            "FunctionContentsGrammar",
+                            ephemeral_name="FunctionContentsGrammar",
+                        ),
                     ),
                 ),
-                OneOf(
-                    Ref("ParameterNameSegment"),
-                    Ref("StarSegment"),
-                ),
-                optional=True,
             ),
-            Ref("PostFunctionGrammar", optional=True),
+            Sequence(
+                Sequence(
+                    Ref(
+                        "FunctionNameSegment",
+                        exclude=OneOf(
+                            Ref("DatePartFunctionNameSegment"),
+                            Ref("NormalizeFunctionNameSegment"),
+                            Ref("ValuesClauseSegment"),
+                        ),
+                    ),
+                    Bracketed(
+                        Ref(
+                            "FunctionContentsGrammar",
+                            # The brackets might be empty for some functions...
+                            optional=True,
+                            ephemeral_name="FunctionContentsGrammar",
+                        )
+                    ),
+                ),
+                # Functions returning ARRAYS in BigQuery can have optional
+                # Array Accessor clauses
+                Ref("ArrayAccessorSegment", optional=True),
+                # Functions returning STRUCTs in BigQuery can have the fields
+                # elements referenced (e.g. ".a"), including wildcards (e.g. ".*")
+                # or multiple nested fields (e.g. ".a.b", or ".a.b.c")
+                Sequence(
+                    Ref("DotSegment"),
+                    AnyNumberOf(
+                        Sequence(
+                            Ref("ParameterNameSegment"),
+                            Ref("DotSegment"),
+                        ),
+                    ),
+                    OneOf(
+                        Ref("ParameterNameSegment"),
+                        Ref("StarSegment"),
+                    ),
+                    optional=True,
+                ),
+                Ref("PostFunctionGrammar", optional=True),
+            ),
         ),
+        allow_gaps=False,
     )
 
 
@@ -535,8 +625,7 @@ class FunctionDefinitionGrammar(ansi.FunctionDefinitionGrammar):
             ),
             Sequence(
                 "LANGUAGE",
-                # Not really a parameter, but best fit for now.
-                Ref("ParameterNameSegment"),
+                Ref("NakedIdentifierSegment"),
                 Sequence(
                     "OPTIONS",
                     Bracketed(
@@ -619,30 +708,8 @@ class DatatypeSegment(ansi.DatatypeSegment):
     match_grammar = OneOf(  # Parameter type
         Ref("DatatypeIdentifierSegment"),  # Simple type
         Sequence("ANY", "TYPE"),  # SQL UDFs can specify this "type"
-        Sequence(
-            "ARRAY",
-            Bracketed(
-                Ref("DatatypeSegment"),
-                bracket_type="angle",
-                bracket_pairs_set="angle_bracket_pairs",
-            ),
-        ),
-        Sequence(
-            "STRUCT",
-            Bracketed(
-                Delimited(  # Comma-separated list of field names/types
-                    Sequence(
-                        Ref("ParameterNameSegment"),
-                        Ref("DatatypeSegment"),
-                        Ref("OptionsSegment", optional=True),
-                    ),
-                    delimiter=Ref("CommaSegment"),
-                    bracket_pairs_set="angle_bracket_pairs",
-                ),
-                bracket_type="angle",
-                bracket_pairs_set="angle_bracket_pairs",
-            ),
-        ),
+        Ref("SimpleArrayTypeGrammar"),
+        Ref("StructTypeGrammar"),
     )
 
 
@@ -704,27 +771,6 @@ class NamedArgumentSegment(BaseSegment):
     )
 
 
-class LiteralCoercionSegment(BaseSegment):
-    """A casting operation with a type name preceding a string literal.
-
-    BigQuery allows string literals to be explicitly coerced to one of the
-    following 4 types:
-    - DATE
-    - DATETIME
-    - TIME
-    - TIMESTAMP
-
-    https://cloud.google.com/bigquery/docs/reference/standard-sql/conversion_rules#literal_coercion
-
-    """
-
-    type = "cast_expression"
-    match_grammar = Sequence(
-        OneOf("DATE", "DATETIME", "TIME", "TIMESTAMP"),
-        Ref("QuotedLiteralSegment"),
-    )
-
-
 # Inherit from the ANSI ObjectReferenceSegment this way so we can inherit
 # other segment types from it.
 class ObjectReferenceSegment(ansi.ObjectReferenceSegment):
@@ -938,8 +984,7 @@ class SetStatementSegment(BaseSegment):
     """
 
     type = "set_segment"
-    match_grammar = StartsWith("SET")
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "SET",
         OneOf(
             Ref("NakedIdentifierSegment"),
@@ -1047,6 +1092,11 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
         "TABLE",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
+        Sequence(
+            OneOf("COPY", "LIKE"),
+            Ref("TableReferenceSegment"),
+            optional=True,
+        ),
         # Column list
         Sequence(
             Bracketed(
@@ -1073,6 +1123,51 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
     )
 
 
+class CreateExternalTableStatementSegment(BaseSegment):
+    """A `CREATE EXTERNAL TABLE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_external_table_statement
+    """
+
+    type = "create_external_table_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        Sequence("OR", "REPLACE", optional=True),
+        "EXTERNAL",
+        "TABLE",
+        Sequence("IF", "NOT", "EXISTS", optional=True),
+        Ref("TableReferenceSegment"),
+        Bracketed(
+            Delimited(
+                Ref("ColumnDefinitionSegment"),
+                allow_trailing=True,
+            ),
+            optional=True,
+        ),
+        # Although not specified in the BigQuery documentation optinal arguments for
+        # CREATE EXTERNAL TABLE statements can be ordered arbitrarily.
+        AnyNumberOf(
+            # connection names have the same rules as table names in BigQuery
+            Sequence("WITH", "CONNECTION", Ref("TableReferenceSegment"), optional=True),
+            Sequence(
+                "WITH",
+                "PARTITION",
+                "COLUMNS",
+                Bracketed(
+                    Delimited(
+                        Ref("ColumnDefinitionSegment"),
+                        allow_trailing=True,
+                    ),
+                    optional=True,
+                ),
+                optional=True,
+            ),
+            Ref("OptionsSegment", optional=True),
+        ),
+    )
+
+
 class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
     """A `CREATE VIEW` statement.
 
@@ -1110,9 +1205,7 @@ class FromPivotExpressionSegment(BaseSegment):
     """
 
     type = "from_pivot_expression"
-    match_grammar = Sequence("PIVOT", Bracketed(Anything()))
-
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "PIVOT",
         Bracketed(
             Delimited(
@@ -1209,13 +1302,20 @@ class InsertStatementSegment(ansi.InsertStatementSegment):
     N.B. not a complete implementation.
     """
 
-    match_grammar = ansi.InsertStatementSegment.match_grammar
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "INSERT",
         Ref.keyword("INTO", optional=True),
         Ref("TableReferenceSegment"),
-        Ref("BracketedColumnReferenceListGrammar", optional=True),
-        Ref("SelectableGrammar"),
+        OneOf(
+            # As SelectableGrammar can be bracketed too, the parse gets confused
+            # so we need slightly odd syntax here to allow those to parse (rather
+            # than just add optional=True to BracketedColumnReferenceListGrammar).
+            Ref("SelectableGrammar"),
+            Sequence(
+                Ref("BracketedColumnReferenceListGrammar"),
+                Ref("SelectableGrammar"),
+            ),
+        ),
     )
 
 
@@ -1309,3 +1409,104 @@ class MergeInsertClauseSegment(ansi.MergeInsertClauseSegment):
         ),
         Sequence("INSERT", "ROW"),
     )
+
+
+class DeleteStatementSegment(BaseSegment):
+    """A `DELETE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/dml-syntax#delete_statement
+    """
+
+    type = "delete_statement"
+    # match grammar. This one makes sense in the context of knowing that it's
+    # definitely a statement, we just don't know what type yet.
+    match_grammar: Matchable = Sequence(
+        "DELETE",
+        Ref.keyword("FROM", optional=True),
+        Ref("ObjectReferenceSegment"),
+        Ref("AliasExpressionSegment", optional=True),
+        Ref("WhereClauseSegment", optional=True),
+    )
+
+
+class ExportStatementSegment(BaseSegment):
+    """`EXPORT` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/other-statements#export_data_statement
+    """
+
+    type = "export_statement"
+    match_grammar: Matchable = Sequence(
+        "EXPORT",
+        "DATA",
+        Sequence("WITH", "CONNECTION", Ref("ObjectReferenceSegment"), optional=True),
+        "OPTIONS",
+        Bracketed(
+            Delimited(
+                # String options
+                # Note: adding as own type, rather than keywords as convention with
+                # Bigquery, as per the docs, is to put Keywords in uppercase, and these
+                # in lowercase.
+                Sequence(
+                    OneOf(
+                        StringParser(
+                            "compression",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                        StringParser(
+                            "field_delimiter",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                        StringParser(
+                            "format",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                        StringParser(
+                            "uri",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                    ),
+                    Ref("EqualsSegment"),
+                    Ref("QuotedLiteralSegment"),
+                ),
+                # Bool options
+                # Note: adding as own type, rather than keywords as convention with
+                # Bigquery, as per the docs, is to put Keywords in uppercase, and these
+                # in lowercase.
+                Sequence(
+                    OneOf(
+                        StringParser(
+                            "header",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                        StringParser(
+                            "overwrite",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                        StringParser(
+                            "use_avro_logical_types",
+                            CodeSegment,
+                            name="export_option",
+                            type="export_option",
+                        ),
+                    ),
+                    Ref("EqualsSegment"),
+                    OneOf("TRUE", "FALSE"),
+                ),
+            ),
+        ),
+        "AS",
+        Ref("SelectableGrammar"),
+    )
