@@ -82,6 +82,7 @@ mysql_dialect.sets("unreserved_keywords").update(
     [
         "QUICK",
         "FAST",
+        "SLOW",
         "MEDIUM",
         "EXTENDED",
         "CHANGED",
@@ -92,6 +93,14 @@ mysql_dialect.sets("unreserved_keywords").update(
         "REPAIR",
         "DUPLICATE",
         "NOW",
+        "ENGINE",
+        "ERROR",
+        "OPTIMIZER_COSTS",
+        "RELAY",
+        "STATUS",
+        "USER_RESOURCES",
+        "CHANNEL",
+        "EXPORT",
     ]
 )
 mysql_dialect.sets("reserved_keywords").update(
@@ -176,7 +185,12 @@ mysql_dialect.replace(
             optional=True,
         ),
         OneOf(
-            Ref("QuotedLiteralSegment"),
+            NamedParser(
+                "single_quote",
+                CodeSegment,
+                name="date_constructor_literal",
+                type="literal",
+            ),
             Ref("NumericLiteralSegment"),
         ),
     ),
@@ -676,7 +690,6 @@ class StatementSegment(ansi.StatementSegment):
             Ref("ResignalSegment"),
             Ref("CursorOpenCloseSegment"),
             Ref("CursorFetchSegment"),
-            Ref("DropFunctionStatementSegment"),
             Ref("DropProcedureStatementSegment"),
             Ref("AlterTableStatementSegment"),
             Ref("RenameTableStatementSegment"),
@@ -690,6 +703,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("OptimizeTableStatementSegment"),
             Ref("UpsertClauseListSegment"),
             Ref("InsertRowAliasSegment"),
+            Ref("FlushStatementSegment"),
         ],
     )
 
@@ -1582,7 +1596,7 @@ class DropFunctionStatementSegment(BaseSegment):
     https://dev.mysql.com/doc/refman/8.0/en/drop-function-loadable.html
     """
 
-    type = "drop_function_ statement"
+    type = "drop_function_statement"
 
     # DROP FUNCTION [IF EXISTS] function_name
     match_grammar = Sequence(
@@ -1827,3 +1841,58 @@ class UpdateStatementSegment(BaseSegment):
         Ref("OrderByClauseSegment", optional=True),
         Ref("LimitClauseSegment", optional=True),
     )
+
+
+class FlushStatementSegment(BaseSegment):
+    """A `Flush` statement.
+
+    As per https://dev.mysql.com/doc/refman/8.0/en/flush.html
+    """
+
+    type = "flush_statement"
+    match_grammar: Matchable = Sequence(
+        "FLUSH",
+        OneOf(
+            "NO_WRITE_TO_BINLOG",
+            "LOCAL",
+            optional=True,
+        ),
+        OneOf(
+            Delimited(
+                Sequence("BINARY", "LOGS"),
+                Sequence("ENGINE", "LOGS"),
+                Sequence("ERROR", "LOGS"),
+                Sequence("GENERAL", "LOGS"),
+                "HOSTS",
+                "LOGS",
+                "PRIVILEGES",
+                "OPTIMIZER_COSTS",
+                Sequence(
+                    "RELAY",
+                    "LOGS",
+                    Sequence(
+                        "FOR", "CHANNEL", Ref("ObjectReferenceSegment"), optional=True
+                    ),
+                ),
+                Sequence("SLOW", "LOGS"),
+                "STATUS",
+                "USER_RESOURCES",
+            ),
+            Sequence(
+                "TABLES",
+                Sequence(
+                    Delimited(Ref("TableReferenceSegment"), terminator="WITH"),
+                    optional=True,
+                ),
+                Sequence("WITH", "READ", "LOCK", optional=True),
+            ),
+            Sequence(
+                "TABLES",
+                Sequence(
+                    Delimited(Ref("TableReferenceSegment"), terminator="FOR"),
+                    optional=False,
+                ),
+                Sequence("FOR", "EXPORT", optional=True),
+            ),
+        ),
+    )
