@@ -15,7 +15,6 @@ from sqlfluff.core.parser import (
     Delimited,
     Nothing,
     OptionallyBracketed,
-    Matchable,
 )
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.dialects.dialect_redshift_keywords import (
@@ -23,6 +22,7 @@ from sqlfluff.dialects.dialect_redshift_keywords import (
     redshift_unreserved_keywords,
 )
 from sqlfluff.dialects import dialect_postgres as postgres
+from sqlfluff.dialects import dialect_ansi as ansi
 
 postgres_dialect = load_raw_dialect("postgres")
 ansi_dialect = load_raw_dialect("ansi")
@@ -159,6 +159,27 @@ redshift_dialect.replace(
         ),
         Ref("AliasExpressionSegment", optional=True),
     ),
+    ColumnReferenceSegment=Delimited(
+        Sequence(
+            ansi.ColumnReferenceSegment,
+            AnyNumberOf(Ref("ArrayAccessorSegment")),
+        ),
+        delimiter=OneOf(
+            Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))
+        ),
+        terminator=OneOf(
+            "ON",
+            "AS",
+            "USING",
+            Ref("CommaSegment"),
+            Ref("CastOperatorSegment"),
+            Ref("BinaryOperatorGrammar"),
+            Ref("ColonSegment"),
+            Ref("DelimiterGrammar"),
+            Ref("JoinLikeClauseGrammar"),
+        ),
+        allow_gaps=False,
+    ),
 )
 
 
@@ -200,43 +221,6 @@ redshift_dialect.add(
 )
 
 
-# need to ignore type due to mypy rules on type variables
-# see https://mypy.readthedocs.io/en/stable/common_issues.html#variables-vs-type-aliases
-# for details
-class ColumnReferenceSegment(ObjectReferenceSegment):
-    """A reference to column, field or alias.
-
-    Adjusted to support column references for Redshift's SUPER data type
-    (https://docs.aws.amazon.com/redshift/latest/dg/super-overview.html), which
-    uses a subset of the PartiQL language (https://partiql.org/) to reference
-    columns.
-    """
-
-    type = "column_reference"
-    match_grammar: Matchable = Delimited(
-        Sequence(
-            Ref("SingleIdentifierGrammar"),
-            AnyNumberOf(Ref("ArrayAccessorSegment")),
-            Ref("TimeZoneGrammar", optional=True),
-        ),
-        delimiter=OneOf(
-            Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))
-        ),
-        terminator=OneOf(
-            "ON",
-            "AS",
-            "USING",
-            Ref("CommaSegment"),
-            Ref("CastOperatorSegment"),
-            Ref("BinaryOperatorGrammar"),
-            Ref("ColonSegment"),
-            Ref("DelimiterGrammar"),
-            Ref("JoinLikeClauseGrammar"),
-        ),
-        allow_gaps=False,
-    )
-
-
 class FromUnpivotExpressionSegment(BaseSegment):
     """An UNPIVOT expression.
 
@@ -373,10 +357,7 @@ class DatatypeSegment(BaseSegment):
             "BPCHAR",
             "TEXT",
         ),
-        Sequence(
-            Ref("DateTimeTypeIdentifier"),
-            Ref("TimeZoneGrammar", optional=True),
-        ),
+        Ref("DateTimeTypeIdentifier"),
         # INTERVAL is a data type *only* for conversion operations
         "INTERVAL",
         # boolean types
@@ -642,8 +623,11 @@ class CreateTableStatementSegment(BaseSegment):
                     Sequence(
                         Ref("ColumnReferenceSegment"),
                         Ref("DatatypeSegment"),
-                        AnyNumberOf(Ref("ColumnAttributeSegment"), optional=True),
-                        AnyNumberOf(Ref("ColumnConstraintSegment"), optional=True),
+                        AnyNumberOf(
+                            Ref("ColumnAttributeSegment"),
+                            Ref("ColumnConstraintSegment"),
+                            optional=True,
+                        ),
                     ),
                     Ref("TableConstraintSegment", optional=True),
                 ),
@@ -1253,7 +1237,7 @@ class CopyStatementSegment(postgres.CopyStatementSegment):
             Sequence(
                 "ACCEPTINVCHARS",
                 Ref.keyword("AS", optional=True),
-                Ref("QuotedLiteralSegment"),
+                Ref("QuotedLiteralSegment", optional=True),
                 optional=True,
             ),
             Ref.keyword("BLANKSASNULL", optional=True),
@@ -1285,7 +1269,7 @@ class CopyStatementSegment(postgres.CopyStatementSegment):
             Sequence(
                 "IGNOREHEADER",
                 Ref.keyword("AS", optional=True),
-                Ref("QuotedLiteralSegment"),
+                Ref("LiteralGrammar"),
                 optional=True,
             ),
             Sequence(
@@ -1762,7 +1746,6 @@ class StatementSegment(postgres.StatementSegment):
     parse_grammar = postgres.StatementSegment.parse_grammar.copy(
         insert=[
             Ref("CreateLibraryStatementSegment"),
-            Ref("CreateUserStatementSegment"),
             Ref("CreateGroupStatementSegment"),
             Ref("AlterUserStatementSegment"),
             Ref("AlterGroupStatementSegment"),
@@ -1837,14 +1820,12 @@ class RowFormatDelimitedSegment(BaseSegment):
     )
 
 
-class CreateUserStatementSegment(BaseSegment):
+class CreateUserStatementSegment(ansi.CreateUserStatementSegment):
     """`CREATE USER` statement.
 
     https://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_USER.html
     """
 
-    type = "create_user"
-
     match_grammar = Sequence(
         "CREATE",
         "USER",
@@ -1917,7 +1898,7 @@ class AlterUserStatementSegment(BaseSegment):
     https://docs.aws.amazon.com/redshift/latest/dg/r_ALTER_USER.html
     """
 
-    type = "alter_user"
+    type = "alter_user_statement"
 
     match_grammar = Sequence(
         "ALTER",
