@@ -143,21 +143,30 @@ sparksql_dialect.sets("datetime_units").clear()
 sparksql_dialect.sets("datetime_units").update(
     [
         "YEAR",
-        # Alternate syntax for YEAR
+        "YEARS",
         "YYYY",
         "YY",
         "QUARTER",
+        "QUARTERS",
         "MONTH",
-        # Alternate syntax for MONTH
+        "MONTHS",
         "MON",
         "MM",
         "WEEK",
+        "WEEKS",
         "DAY",
-        # Alternate syntax for DAY
+        "DAYS",
         "DD",
         "HOUR",
+        "HOURS",
         "MINUTE",
+        "MINUTES",
         "SECOND",
+        "SECONDS",
+        "MILLISECOND",
+        "MILLISECONDS",
+        "MICROSECOND",
+        "MICROSECONDS",
     ]
 )
 
@@ -508,6 +517,22 @@ sparksql_dialect.add(
         type="literal",
         trim_chars="@",
     ),
+    # This is the same as QuotedLiteralSegment but
+    # is given a different `name` to stop L048 flagging
+    SignedQuotedLiteralSegment=OneOf(
+        NamedParser(
+            "single_quote",
+            CodeSegment,
+            name="signed_quoted_literal",
+            type="literal",
+        ),
+        NamedParser(
+            "double_quote",
+            CodeSegment,
+            name="signed_quoted_literal",
+            type="literal",
+        ),
+    ),
 )
 
 # Adding Hint related grammar before comment `block_comment` and
@@ -850,14 +875,6 @@ class CreateFunctionStatementSegment(ansi.CreateFunctionStatementSegment):
     """
 
     match_grammar = Sequence(
-        "CREATE",
-        Sequence("OR", "REPLACE", optional=True),
-        Ref("TemporaryGrammar", optional=True),
-        "FUNCTION",
-        Anything(),
-    )
-
-    parse_grammar = Sequence(
         "CREATE",
         Sequence("OR", "REPLACE", optional=True),
         Ref("TemporaryGrammar", optional=True),
@@ -2211,7 +2228,6 @@ class StatementSegment(ansi.StatementSegment):
             Ref("AlterTableStatementSegment"),
             Ref("AlterViewStatementSegment"),
             Ref("CreateHiveFormatTableStatementSegment"),
-            Ref("DropFunctionStatementSegment"),
             Ref("MsckRepairTableStatementSegment"),
             Ref("UseDatabaseStatementSegment"),
             # Auxiliary Statements
@@ -2239,7 +2255,6 @@ class StatementSegment(ansi.StatementSegment):
             Ref("TransactionStatementSegment"),
             Ref("CreateSchemaStatementSegment"),
             Ref("SetSchemaStatementSegment"),
-            Ref("CreateExtensionStatementSegment"),
             Ref("CreateModelStatementSegment"),
             Ref("DropModelStatementSegment"),
         ],
@@ -2338,6 +2353,7 @@ class AliasExpressionSegment(ansi.AliasExpressionSegment):
                 Ref("JoinTypeKeywords"),
                 "WINDOW",
                 "PIVOT",
+                Ref("DatetimeUnitSegment"),
             ),
         ),
     )
@@ -2548,3 +2564,68 @@ class MergeInsertClauseSegment(ansi.MergeInsertClauseSegment):
             ),
         ),
     )
+
+
+class UpdateStatementSegment(ansi.UpdateStatementSegment):
+    """An `Update` statement.
+
+    Enhancing from ANSI dialect to be SparkSQL & Delta Lake specific.
+
+    https://docs.delta.io/latest/delta-update.html#update-a-table
+    """
+
+    match_grammar: Matchable = Sequence(
+        "UPDATE",
+        OneOf(
+            Ref("FileReferenceSegment"),
+            Ref("TableReferenceSegment"),
+        ),
+        # SET is not a resevered word in all dialects (e.g. RedShift)
+        # So specifically exclude as an allowed implict alias to avoid parsing errors
+        Ref(
+            "AliasExpressionSegment",
+            exclude=Ref.keyword("SET"),
+            optional=True,
+        ),
+        Ref("SetClauseListSegment"),
+        Ref("WhereClauseSegment"),
+    )
+
+
+class IntervalLiteralSegment(BaseSegment):
+    """An interval literal segment.
+
+    https://spark.apache.org/docs/latest/sql-ref-literals.html#interval-literal
+    """
+
+    type = "interval_literal"
+
+    match_grammar: Matchable = Sequence(
+        Ref("SignedSegmentGrammar", optional=True),
+        OneOf(
+            Ref("NumericLiteralSegment"),
+            Ref("SignedQuotedLiteralSegment"),
+        ),
+        Ref("DatetimeUnitSegment"),
+        Ref.keyword("TO", optional=True),
+        Ref("DatetimeUnitSegment", optional=True),
+    )
+
+
+class IntervalExpressionSegment(ansi.IntervalExpressionSegment):
+    """An interval expression segment.
+
+    Redefining from ANSI dialect to allow for additional syntax.
+
+    https://spark.apache.org/docs/latest/sql-ref-literals.html#interval-literal
+    """
+
+    match_grammar: Matchable = Sequence(
+        "INTERVAL",
+        OneOf(
+            AnyNumberOf(
+                Ref("IntervalLiteralSegment"),
+            ),
+            Ref("QuotedLiteralSegment"),
+        ),
+    )
