@@ -56,6 +56,7 @@ RESERVED_KEYWORDS = [
     "DISTRIBUTED",
     "DOUBLE",
     "DROP",
+    "DYNAMIC",
     "ELSE",
     "END",
     "ERRLVL",
@@ -66,16 +67,19 @@ RESERVED_KEYWORDS = [
     "EXISTS",
     "EXIT",
     "EXTERNAL",
+    "FAST_FORWARD",
     "FETCH",
     "FILE",
     "FILLFACTOR",
     "FOR",
+    "FORWARD_ONLY",
     "FOREIGN",
     "FREETEXT",
     "FREETEXTTABLE",
     "FROM",
     "FULL",
     "FUNCTION",
+    "GLOBAL",
     "GO",
     "GOTO",
     "GRANT",
@@ -95,10 +99,12 @@ RESERVED_KEYWORDS = [
     "IS",
     "JOIN",
     "KEY",
+    "KEYSET",
     "KILL",
     "LEFT",
     "LIKE",
     "LINENO",
+    "LOCAL",
     "MERGE",
     "NATIONAL",
     "NATIVE_COMPILATION",
@@ -116,6 +122,7 @@ RESERVED_KEYWORDS = [
     "OPENQUERY",
     "OPENROWSET",
     "OPENXML",
+    "OPTIMISTIC",
     "OPTION",
     "OR",
     "ORDER",
@@ -131,6 +138,7 @@ RESERVED_KEYWORDS = [
     "PUBLIC",
     "RAISERROR",
     "READ",
+    "READ_ONLY",
     "READTEXT",
     "RECONFIGURE",
     "REFERENCES",
@@ -147,6 +155,8 @@ RESERVED_KEYWORDS = [
     "RULE",
     "SAVE",
     "SCHEMA",
+    "SCROLL",
+    "SCROLL_LOCKS",
     "SELECT",
     "SEMANTICKEYPHRASETABLE",
     "SEMANTICSIMILARITYDETAILSTABLE",
@@ -156,6 +166,7 @@ RESERVED_KEYWORDS = [
     "SETUSER",
     "SHUTDOWN",
     "SOME",
+    "STATIC",
     "STATISTICS",
     "SYSTEM_USER",
     "TABLE",
@@ -171,6 +182,7 @@ RESERVED_KEYWORDS = [
     "TRUNCATE",
     "TRY_CONVERT",
     "TSEQUAL",
+    "TYPE_WARNING",
     "UNION",
     "UNIQUE",
     "UNPIVOT",
@@ -206,6 +218,8 @@ UNRESERVED_KEYWORDS = [
     "APPLY",
     "ARITHABORT",
     "ARITHIGNORE",
+    "AT",
+    "AUTO",
     "BERNOULLI",
     "BLOCKERS",
     "BREAK",
@@ -221,6 +235,7 @@ UNRESERVED_KEYWORDS = [
     "CONCAT",
     "CONCAT_NULL_YIELDS_NULL",
     "CONTINUE",
+    "CONTROL",
     "COMPRESSION_DELAY",
     "CURSOR_CLOSE_ON_COMMIT",
     "CYCLE",
@@ -242,14 +257,17 @@ UNRESERVED_KEYWORDS = [
     "ENCRYPTION_TYPE",
     "EXPAND",
     "EXPLAIN",  # Azure Synapse Analytics specific
+    "EXPLICIT",
     "EXTERNALPUSHDOWN",
     "FAST",
     "FILESTREAM",
     "FILESTREAM_ON",
     "FILTER",
     "FIPS_FLAGGER",
+    "FIRST",
     "FMTONLY",
     "FOLLOWING",
+    "FOR",
     "FORCE",
     "FORCEPLAN",
     "FORCESCAN",
@@ -280,16 +298,19 @@ UNRESERVED_KEYWORDS = [
     # but could break TSQL parsing to add there
     "LABEL",
     "LANGUAGE",
+    "LAST",
     "LEVEL",
     "LOAD",  # listed as reserved but functionally unreserved
     "LOCATION",
     "LOCK_TIMEOUT",
     "LOG",
+    "LOGIN",
     "LOOP",
     "MAX_DURATION",
     "MAX_GRANT_PERCENT",
     "MASKED",
     "MATCHED",
+    "MAX",
     "MAXDOP",
     "MAXRECURSION",
     "MAXVALUE",
@@ -323,6 +344,7 @@ UNRESERVED_KEYWORDS = [
     "PARSEONLY",
     "PARTITION",
     "PARTITIONS",
+    "PATH",
     "PERCENTILE_CONT",
     "PERCENTILE_DISC",
     "PERSISTED",
@@ -336,11 +358,13 @@ UNRESERVED_KEYWORDS = [
     "RANDOMIZED",
     "RANGE",
     "RANK",
+    "RAW",
     "READCOMMITTED",
     "READCOMMITTEDLOCK",
     "READONLY",
     "READPAST",
     "READUNCOMMITTED",
+    "RECEIVE",
     "RECOMPILE",
     "RECURSIVE",
     "REMOTE_PROC_TRANSACTIONS",
@@ -387,6 +411,7 @@ UNRESERVED_KEYWORDS = [
     "SYSTEM",
     "TABLOCK",
     "TABLOCKX",
+    "TAKE",
     "TARGET",
     "TEXTIMAGE_ON",
     "THROW",
@@ -414,4 +439,5 @@ UNRESERVED_KEYWORDS = [
     "XACT_ABORT",
     "XLOCK",
     "XML",
+    "ZONE",
 ]
