@@ -1,10 +1,12 @@
 """Implementation of Rule L005."""
 from typing import Optional
 
+from sqlfluff.core.parser import RawSegment
 from sqlfluff.core.rules.base import BaseRule, LintResult, LintFix, RuleContext
-from sqlfluff.core.rules.doc_decorators import document_fix_compatible
+from sqlfluff.core.rules.doc_decorators import document_fix_compatible, document_groups
 
 
+@document_groups
 @document_fix_compatible
 class Rule_L005(BaseRule):
     """Commas should not have whitespace directly before them.
@@ -37,20 +39,18 @@ class Rule_L005(BaseRule):
         FROM foo
     """
 
+    groups = ("all", "core")
+
     def _eval(self, context: RuleContext) -> Optional[LintResult]:
-        """Commas should not have whitespace directly before them.
-
-        We need at least one segment behind us for this to work.
-
-        """
-        if len(context.raw_stack) >= 1:
-            cm1 = context.raw_stack[-1]
-            if (
-                context.segment.is_type("comma")
-                and cm1.is_type("whitespace")
-                and cm1.pos_marker.line_pos > 1
-            ):
-                anchor = cm1
-                return LintResult(anchor=anchor, fixes=[LintFix.delete(cm1)])
-        # Otherwise fine
+        """Commas should not have whitespace directly before them."""
+        anchor: Optional[RawSegment] = context.raw_segment_pre
+        if (
+            # We need at least one segment previous segment for this to work.
+            anchor is not None
+            and context.segment.is_type("comma")
+            and anchor.is_type("whitespace")
+            and anchor.pos_marker.line_pos > 1
+        ):
+            return LintResult(anchor=anchor, fixes=[LintFix.delete(anchor)])
+        # Otherwise fine.
         return None
