@@ -5,12 +5,14 @@ from typing import Tuple, List
 from sqlfluff.core.rules.doc_decorators import (
     document_configuration,
     document_fix_compatible,
+    document_groups,
 )
 from sqlfluff.rules.L010 import Rule_L010
 
 
-@document_configuration
+@document_groups
 @document_fix_compatible
+@document_configuration
 class Rule_L063(Rule_L010):
     """Inconsistent capitalisation of datatypes.
 
@@ -53,6 +55,8 @@ class Rule_L063(Rule_L010):
 
     """
 
+    groups = ("all",)
+    lint_phase = "post"
     _target_elems: List[Tuple[str, str]] = [
         ("parenttype", "data_type"),
         ("parenttype", "datetime_type_identifier"),
@@ -63,5 +67,6 @@ class Rule_L063(Rule_L010):
     config_keywords = [
         "extended_capitalisation_policy",
         "ignore_words",
+        "ignore_words_regex",
     ]
     _description_elem = "Datatypes"
